@@ -8,12 +8,21 @@ set -e
 # since we don't assume the presence of either a C cross-compiler
 # or a host machine on which to run its native compiler.
 
-# FIXME: hppa
-for arch in alpha arm arm64 mips ppc sparc x86 x86-64
+if [ -z "$*" ]; then
+    targets="alpha arm arm64 hppa mips ppc ppc64 sparc x86 x86-64 riscv"
+else
+    targets="$@"
+fi
+
+for arch in $targets
 do
   echo TESTING $arch
   ltf=local-target-features.lisp-expr
-  echo '(lambda (features) (union features (list :crossbuild-test ' > $ltf
+  # Whether any of the :OS-PROVIDES-* features are present is mostly immaterial
+  # to this cross-build test. Some of the provisions are only for C code which
+  # we don't compile. Or else it doesn't matter much which lisp code is compiled.
+  # Assume that dlopen() is provided so that we don't try to read sbcl.nm though.
+  echo '(lambda (features) (union features (list :crossbuild-test :os-provides-dlopen ' > $ltf
   cat crossbuild-runner/backends/$arch/local-target-features >> $ltf
   echo ')))' >> $ltf
 
