@@ -187,6 +187,7 @@
   #+win32 (handle nil :type (or null (signed-byte 32)))
   #-win32
   serve-event-pipe)
+(declaim (freeze-type process))
 
 (defmethod print-object ((process process) stream)
   (print-unreadable-object (process stream :type t)
@@ -1278,7 +1279,7 @@ Users Manual for details about the PROCESS structure.
           (t
            (fail "invalid option: ~S" object))))))
 
-#+(or linux sunos hpux)
+#+(or linux sunos hpux haiku)
 (defun software-version ()
   "Return a string describing version of the supporting software, or NIL
   if not available."
@@ -1286,5 +1287,9 @@ Users Manual for details about the PROCESS structure.
       (setf sb-sys::*software-version*
             (string-trim '(#\newline)
                          (sb-kernel:with-simple-output-to-string (stream)
-                           (run-program "/bin/uname" `("-r")
+                           (run-program "/bin/uname"
+                                        ;; "-r" on haiku just prints "1"
+                                        ;; but "-v" prints some detail.
+                                        #+haiku '("-v")
+                                        #-haiku '("-r")
                                         :output stream))))))
