@@ -18,8 +18,7 @@
 
 ;;; This gets called by LOAD to resolve newly positioned objects
 ;;; with things (like code instructions) that have to refer to them.
-;;; Return :ABSOLUTE if an absolute fixup needs to be recorded in %CODE-FIXUPS,
-;;; and return :RELATIVE if a relative fixup needs to be recorded.
+;;; Return KIND  if the fixup needs to be recorded in %CODE-FIXUPS.
 ;;; The code object we're fixing up is pinned whenever this is called.
 (defun fixup-code-object (code offset fixup kind flavor)
   (declare (type index offset) (ignorable flavor))
@@ -29,29 +28,26 @@
                        (signed-sap-ref-32 sap offset))
                    fixup)))
     (ecase kind
-        (:absolute64
+      (:absolute64
          ;; Word at sap + offset contains a value to be replaced by
          ;; adding that value to fixup.
          (setf (sap-ref-64 sap offset) fixup))
-        (:absolute
+      (:absolute
          ;; Word at sap + offset contains a value to be replaced by
          ;; adding that value to fixup.
          (setf (sap-ref-32 sap offset) fixup))
-        (:relative
+      (:relative
          ;; Fixup is the actual address wanted.
          ;; Replace word with value to add to that loc to get there.
          ;; In the #-immobile-code case, there's nothing to assert.
-         ;; Relative fixups pretty much can't happen.
+         ;; Relative fixups don't exist with movable code.
          #+immobile-code
          (unless (immobile-space-obj-p code)
            (error "Can't compute fixup relative to movable object ~S" code))
          (setf (signed-sap-ref-32 sap offset)
-               (etypecase fixup
-                 (integer
-                  ;; JMP/CALL are relative to the next instruction,
-                  ;; so add 4 bytes for the size of the displacement itself.
-                  (- fixup
-                     (the (unsigned-byte 64) (+ (sap-int sap) offset 4)))))))))
+               ;; JMP/CALL are relative to the next instruction,
+               ;; so add 4 bytes for the size of the displacement itself.
+               (- fixup (sap-int (sap+ sap (+ offset 4))))))))
   ;; An absolute fixup is stored in the code header's %FIXUPS slot if it
   ;; references an immobile-space (but not static-space) object.
   ;; Note that:
@@ -182,7 +178,7 @@
 (defconstant trampoline-entry-offset n-word-bytes)
 (defun make-simplifying-trampoline (fun)
   (let ((code (truly-the (values code-component &optional)
-                         (allocate-code-object :dynamic 3 24)))) ; KLUDGE
+                         (allocate-code-object :dynamic 0 3 24)))) ; KLUDGE
     (setf (%code-debug-info code) fun)
     (let ((sap (sap+ (code-instructions code) trampoline-entry-offset))
           (ea (+ (logandc2 (get-lisp-obj-address code) lowtag-mask)
@@ -206,6 +202,8 @@
                      (setf (sap-ref-32 sap 7) (logior (ash disp8 24) #x408B48))
                      11))))
         (setf (sap-ref-32 sap i) #xFD60FF))) ; JMP [RAX-3]
+    ;; Verify that the jump table size reads as  0.
+    (aver (zerop (code-jump-table-words code)))
     ;; It is critical that there be a trailing 'uint16' of 0 in this object
     ;; so that CODE-N-ENTRIES reports 0.  By luck, there is exactly enough
     ;; room in the object to hold two 0 bytes. It would be easy enough to enlarge
@@ -220,7 +218,7 @@
     (or (and (eql kind sb-vm:funcallable-instance-widetag)
              ;; if the FIN has no raw words then it has no internal trampoline
              (eql (layout-bitmap (%funcallable-instance-layout fun))
-                  sb-kernel::+layout-all-tagged+))
+                  sb-kernel:+layout-all-tagged+))
         (eql kind sb-vm:closure-widetag))))
 
 (defconstant +fsc-layout-bitmap+
