@@ -10,7 +10,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!VM")
+(in-package "SB-VM")
 
 ;;;; data object ref/set stuff
 
@@ -25,18 +25,20 @@
 (define-vop (set-slot)
   (:args (object :scs (descriptor-reg))
          (value :scs (descriptor-reg any-reg null zero)))
-  (:info name offset lowtag #!+gengc remember)
+  (:info name offset lowtag #+gengc remember)
   (:ignore name)
   (:results)
   (:generator 1
-    #!+gengc
+    #+gengc
     (if remember
         (storew-and-remember-slot value object offset lowtag)
         (storew value object offset lowtag))
-    #!-gengc
+    #-gengc
     (storew value object offset lowtag)))
 
-(define-vop (init-slot set-slot))
+(define-vop (init-slot set-slot)
+  (:info name dx-p offset lowtag)
+  (:ignore name dx-p))
 
 ;;;; symbol hacking VOPs
 
@@ -57,7 +59,7 @@
 ;;; With SYMBOL-VALUE, we check that the value isn't the trap object.
 ;;; So SYMBOL-VALUE of NIL is NIL.
 (define-vop (symbol-value checked-cell-ref)
-  (:translate symbol-value)
+  (:translate symeval)
   (:generator 9
     (move object obj-temp)
     (loadw value obj-temp symbol-value-slot other-pointer-lowtag)
@@ -87,7 +89,7 @@
 (define-vop (fast-symbol-value cell-ref)
   (:variant symbol-value-slot other-pointer-lowtag)
   (:policy :fast)
-  (:translate symbol-value))
+  (:translate symeval))
 
 (define-vop (symbol-hash)
   (:policy :fast-safe)
@@ -106,9 +108,9 @@
 ;;; On unithreaded builds these are just copies of the non-global versions.
 (define-vop (%set-symbol-global-value set))
 (define-vop (symbol-global-value symbol-value)
-  (:translate symbol-global-value))
+  (:translate sym-global-val))
 (define-vop (fast-symbol-global-value fast-symbol-value)
-  (:translate symbol-global-value))
+  (:translate sym-global-val))
 
 ;;;; fdefinition (FDEFN) objects
 
@@ -144,7 +146,7 @@
       (load-type type function (- fun-pointer-lowtag))
       (inst xor type simple-fun-widetag type)
       (inst addq function
-            (- (ash simple-fun-code-offset word-shift) fun-pointer-lowtag)
+            (- (ash simple-fun-insts-offset word-shift) fun-pointer-lowtag)
             lip)
       (inst beq type normal-fn)
       (inst li (make-fixup 'closure-tramp :assembly-routine) lip)
@@ -182,7 +184,7 @@
     (inst addq bsp-tn (* 2 n-word-bytes) bsp-tn)
     (storew temp bsp-tn (- binding-value-slot binding-size))
     (storew symbol bsp-tn (- binding-symbol-slot binding-size))
-    (#!+gengc storew-and-remember-slot #!-gengc storew
+    (#+gengc storew-and-remember-slot #-gengc storew
              val symbol symbol-value-slot other-pointer-lowtag)))
 
 
@@ -191,7 +193,7 @@
   (:generator 0
     (loadw symbol bsp-tn (- binding-symbol-slot binding-size))
     (loadw value bsp-tn (- binding-value-slot binding-size))
-    (#!+gengc storew-and-remember-slot #!-gengc storew
+    (#+gengc storew-and-remember-slot #-gengc storew
              value symbol symbol-value-slot other-pointer-lowtag)
     (storew zero-tn bsp-tn (- binding-symbol-slot binding-size))
     (storew zero-tn bsp-tn (- binding-value-slot binding-size))
@@ -215,7 +217,7 @@
       (loadw symbol bsp-tn (- binding-symbol-slot binding-size))
       (loadw value bsp-tn (- binding-value-slot binding-size))
       (inst beq symbol skip)
-      (#!+gengc storew-and-remember-slot #!-gengc storew
+      (#+gengc storew-and-remember-slot #-gengc storew
                value symbol symbol-value-slot other-pointer-lowtag)
       (storew zero-tn bsp-tn (- binding-symbol-slot binding-size))
 
@@ -241,11 +243,19 @@
   funcallable-instance-info-offset fun-pointer-lowtag
   (descriptor-reg any-reg) * %funcallable-instance-info)
 
-(define-vop (closure-ref slot-ref)
-  (:variant closure-info-offset fun-pointer-lowtag))
+(define-vop (closure-ref)
+  (:args (object :scs (descriptor-reg)))
+  (:results (value :scs (descriptor-reg any-reg)))
+  (:info offset)
+  (:generator 4
+    (loadw value object (+ closure-info-offset offset) fun-pointer-lowtag)))
 
-(define-vop (closure-init slot-set)
-  (:variant closure-info-offset fun-pointer-lowtag))
+(define-vop (closure-init)
+  (:args (object :scs (descriptor-reg))
+         (value :scs (descriptor-reg any-reg)))
+  (:info offset)
+  (:generator 4
+    (storew value object (+ closure-info-offset offset) fun-pointer-lowtag)))
 
 (define-vop (closure-init-from-fp)
   (:args (object :scs (descriptor-reg)))
@@ -263,7 +273,7 @@
 
 ;;;; instance hackery
 
-(define-vop (instance-length)
+(define-vop ()
   (:policy :fast-safe)
   (:translate %instance-length)
   (:args (struct :scs (descriptor-reg)))
@@ -289,7 +299,7 @@
 
 ;;;; mutator accessing
 
-#!+gengc
+#+gengc
 (progn
 
 (eval-when (:compile-toplevel :load-toplevel :execute)
@@ -297,7 +307,7 @@
   ;; would probably be nice to restore GENGC support so that the Alpha
   ;; doesn't have to crawl along with stop'n'copy. When we do, the CMU
   ;; CL code below will need updating to the SBCL way of looking at
-  ;; things, e.g. at least using "SB-KERNEL" or "SB!KERNEL" instead of
+  ;; things, e.g. at least using "SB-KERNEL" or "SB-KERNEL" instead of
   ;; :KERNEL. -- WHN 2001-05-08
   (error "This code is stale as of sbcl-0.6.12."))
 
