@@ -12,7 +12,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!C")
+(in-package "SB-C")
 
 (/show0 "knownfun.lisp 17")
 
@@ -76,7 +76,11 @@
                        overwrite-fndb-silently
                        call-type-deriver
                        annotation)
-  (let ((ctype (specifier-type type)))
+  (let* ((ctype (specifier-type type))
+         (type-to-store (if (contains-unknown-type-p ctype)
+                            ;; unparse it, so SFUNCTION -> FUNCTION
+                            (type-specifier ctype)
+                            ctype)))
     (dolist (name names)
       (unless overwrite-fndb-silently
         (let ((old-fun-info (info :function :info name)))
@@ -93,7 +97,7 @@
             (cerror "Go ahead, overwrite it."
                     "~@<overwriting old FUN-INFO ~2I~_~S ~I~_for ~S~:>"
                     old-fun-info name))))
-      (setf (info :function :type name) ctype)
+      (setf (info :function :type name) type-to-store)
       (setf (info :function :where-from name) :declared)
       (setf (info :function :kind name) :function)
       (setf (info :function :info name)
@@ -179,13 +183,17 @@
           (labels ((annotation-p (x)
                      (typep x '(or (cons (member function function-designator modifying
                                           inhibit-flushing))
-                                (member type-specifier))))
+                                (member type-specifier proper-sequence proper-list
+                                 proper-or-dotted-list proper-or-circular-list))))
                    (strip-annotation (x)
                      (if (consp x)
                          (ecase (car x)
                            ((function function-designator) (car x))
                            ((modifying inhibit-flushing) (cadr x)))
-                         x))
+                         (case x
+                           (proper-sequence 'sequence)
+                           ((proper-list proper-or-dotted-list proper-or-circular-list) 'list)
+                           (t x))))
                    (process-positional (type)
                      (incf i)
                      (cond ((annotation-p type)
@@ -247,8 +255,7 @@
       (let* ((sym (lvar-value lvar))
              (var (maybe-find-free-var sym))
              (local-type (when var
-                           (let ((*lexenv* (node-lexenv node)))
-                             (lexenv-find var type-restrictions))))
+                           (lexenv-find var type-restrictions :lexenv (node-lexenv node))))
              (global-type (info :variable :type sym)))
         (if local-type
             (type-intersection local-type global-type)
@@ -266,6 +273,11 @@
   (let ((lvar (car (last (combination-args call)))))
     (when lvar (lvar-type lvar))))
 
+(defun result-type-nth-arg (n)
+  (lambda (call)
+    (let ((lvar (nth n (combination-args call))))
+      (when lvar (lvar-type lvar)))))
+
 ;;; Derive the result type according to the float contagion rules, but
 ;;; always return a float. This is used for irrational functions that
 ;;; preserve realness of their arguments.
@@ -396,8 +408,12 @@
 
 ;;; Return MAX MIN
 (defun sequence-lvar-dimensions (lvar)
-  (if (not (constant-lvar-p lvar))
-      (let ((max 0) (min array-total-size-limit))
+  (if (constant-lvar-p lvar)
+      (let ((value (lvar-value lvar)))
+        (and (proper-sequence-p value)
+             (let ((length (length value)))
+               (values length length))))
+      (let ((max 0) (min sb-xc:array-total-size-limit))
         (block nil
           (labels ((max-dim (type)
                      ;; This can deal with just enough hair to handle type STRING,
@@ -424,11 +440,7 @@
             ;; However that's probably not an important use, so the above
             ;; logic restricts itself to simple arrays.
             (max-dim (lvar-type lvar))
-            (values max min))))
-      (let ((value (lvar-value lvar)))
-        (and (typep value 'sequence)
-             (let ((length (length value)))
-               (values length length))))))
+            (values max min))))))
 
 (defun position-derive-type (call)
   (let ((dim (sequence-lvar-dimensions (second (combination-args call)))))
@@ -443,31 +455,53 @@
 ;;; This used to be done in DEFOPTIMIZER DERIVE-TYPE, but
 ;;; ASSERT-CALL-TYPE already asserts the ARRAY type, so it gets an extra
 ;;; assertion that may not get eliminated and requires extra work.
-(defun array-call-type-deriver (call trusted &optional set)
+(defun array-call-type-deriver (call trusted &optional set row-major-aref)
   (let* ((fun (combination-fun call))
          (type (lvar-fun-type fun))
          (policy (lexenv-policy (node-lexenv call)))
          (args (combination-args call)))
     (when (fun-type-p type)
-      (flet ((assert-type (arg type &optional set)
-               (when (if set
-                         (assert-modifying-lvar-type arg type
-                                                     (lvar-fun-name fun)
-                                                     policy)
-                         (assert-lvar-type arg type policy))
+      (flet ((assert-type (arg type &optional set index)
+               (when (cond (index
+                            (assert-array-index-lvar-type arg type policy))
+                           (t
+                            (when set
+                              (add-annotation arg
+                                              (make-lvar-modified-annotation :caller (lvar-fun-name fun))))
+                            (assert-lvar-type arg type policy)))
                  (unless trusted (reoptimize-lvar arg)))))
         (let ((required (fun-type-required type)))
           (when set
             (assert-type (pop args)
                          (pop required)))
           (assert-type (pop args)
-                       (type-intersection
-                        (pop required)
-                        (specifier-type `(array * ,(length args))))
+                       (if row-major-aref
+                           (pop required)
+                           (type-intersection
+                            (pop required)
+                            (specifier-type `(array * ,(length args)))))
                        set)
           (loop for type in required
-                do (assert-type (pop args) type))
+                do
+                (assert-type (pop args) type nil (or (not (and set row-major-aref))
+                                                     args)))
           (loop for type in (fun-type-optional type)
-                do (assert-type (pop args) type))
+                do (assert-type (pop args) type nil t))
           (loop for subscript in args
-                do (assert-type subscript (fun-type-rest type))))))))
+                do (assert-type subscript (fun-type-rest type) nil t)))))))
+
+(defun append-call-type-deriver (call trusted)
+  (let* ((policy (lexenv-policy (node-lexenv call)))
+         (args (combination-args call))
+         (list-type (specifier-type 'list)))
+    ;; All but the last argument should be proper lists
+    (loop for (arg next) on args
+          while next
+          do
+          (add-annotation
+           arg
+           (make-lvar-proper-sequence-annotation
+            :kind 'proper-list))
+          (when (and (assert-lvar-type arg list-type policy)
+                     (not trusted))
+            (reoptimize-lvar arg)))))
