@@ -223,7 +223,6 @@
   (:generator 2
     (inst li res (make-fixup foreign-symbol :foreign))))
 
-#+linkage-table
 (define-vop (foreign-symbol-dataref-sap)
   (:translate foreign-symbol-dataref-sap)
   (:policy :fast-safe)
@@ -252,7 +251,9 @@
     (let ((cur-nfp (current-nfp-tn vop)))
       (when cur-nfp
         (store-stack-tn nfp-save cur-nfp))
-      (load-symbol-value tramp call-into-c)
+      ;; (linkage-table-entry-address 0) is "call-into-c" in mips-assem.S
+      (inst lw tramp null-tn (- (linkage-table-entry-address 0) nil-value))
+      (inst nop)
       (inst jal tramp)
       (inst move cfunc function)
       (when cur-nfp
