@@ -165,15 +165,14 @@
 ;;; because a temp register is needed to do inline allocation.
 ;;; TEMP-TN, in this case, can be any register, since it holds a
 ;;; double-word aligned address (essentially a fixnum).
-(defun allocation (result-tn size lowtag &key stack-p node temp-tn flag-tn)
+(defun allocation (type size lowtag result-tn &key stack-p node temp-tn flag-tn)
   ;; We assume we're in a pseudo-atomic so the pseudo-atomic bit is
   ;; set.  If the lowtag also has a 1 bit in the same position, we're all
   ;; set.  Otherwise, we need to zap out the lowtag from alloc-tn, and
   ;; then or in the lowtag.
   ;; Normal allocation to the heap.
   (declare (ignore stack-p node)
-           #-gencgc
-           (ignore temp-tn flag-tn))
+           (ignorable type temp-tn flag-tn))
   #-gencgc
   (progn
          (if (logbitp (1- n-lowtag-bits) lowtag)
@@ -222,7 +221,9 @@
          ;; the actual end of the region?  If so, we need a full alloc.
          ;; The C code depends on this exact form of instruction.  If
          ;; either changes, you have to change the other appropriately!
-         (inst tw :lgt result-tn flag-tn)
+         ;; See the ppc64 file for more explanation about this.
+         ;; (Or better yet, merge the two codebases)
+         (inst tw (if (eq type 'list) :lgt :lge) result-tn flag-tn)
 
          ;; The C code depends on this instruction sequence taking up
          ;; one machine instruction.
@@ -258,7 +259,7 @@
              (align-csp ,temp-tn)
              (inst ori ,result-tn csp-tn ,lowtag)
              (inst addi csp-tn csp-tn (pad-data-block ,size)))
-         (allocation ,result-tn (pad-data-block ,size) ,lowtag
+         (allocation nil (pad-data-block ,size) ,lowtag ,result-tn
                      :temp-tn ,temp-tn
                      :flag-tn ,flag-tn))
        (when ,type-code
