@@ -60,27 +60,6 @@
           (ensure-class class))
       class))
 
-;;; interface
-(defun specializer-from-type (type &aux args)
-  ;; Avoid style-warning about compiler-macro being unavailable.
-  (declare (notinline make-instance))
-  (when (symbolp type)
-    (return-from specializer-from-type (find-class type)))
-  (when (consp type)
-    (setq args (cdr type) type (car type)))
-  (cond ((symbolp type)
-         (or (ecase type
-               (class    (coerce-to-class (car args)))
-               (prototype (make-instance 'class-prototype-specializer
-                                         :object (coerce-to-class (car args))))
-               (class-eq (class-eq-specializer (coerce-to-class (car args))))
-               (eql      (intern-eql-specializer (car args))))))
-        ;; FIXME: do we still need this?
-        ((and (null args) (typep type 'classoid))
-         (or (classoid-pcl-class type)
-             (ensure-non-standard-class (classoid-name type) type)))
-        ((specializerp type) type)))
-
 ;;; interface
 (defun type-from-specializer (specl)
   (cond ((eq specl t)
@@ -100,7 +79,6 @@
          (error "~S is neither a type nor a specializer." specl))))
 
 (defun type-class (type)
-  (declare (special *the-class-t*))
   (setq type (type-from-specializer type))
   (if (atom type)
       (if (eq type t)
@@ -179,15 +157,11 @@
               (t
                (subtypep (convert-to-system-type type1)
                          (convert-to-system-type type2))))))))
-
-(defvar *built-in-class-symbols* ())
 
-(defun get-built-in-class-symbol (class-name)
-  (or (cadr (assq class-name *built-in-class-symbols*))
-      (let ((symbol (make-class-symbol class-name)))
-        (push (list class-name symbol) *built-in-class-symbols*)
-        symbol)))
-
+(defun make-class-symbol (class-name)
+  (format-symbol #.(find-package "SB-PCL")
+                 "*THE-CLASS-~A*" (symbol-name class-name)))
+
 (defvar *standard-method-combination*)
 (defvar *or-method-combination*)
 
@@ -203,10 +177,10 @@
 
 ;;;; built-in classes
 
-;;; Grovel over SB-KERNEL::*!BUILT-IN-CLASSES* in order to set
+;;; Grovel over SB-KERNEL::+!BUILT-IN-CLASSES+ in order to set
 ;;; SB-PCL:*BUILT-IN-CLASSES*.
 (/show "about to set up SB-PCL::*BUILT-IN-CLASSES*")
-(defvar *built-in-classes*
+(define-load-time-global *built-in-classes*
   (labels ((direct-supers (class)
              (/noshow "entering DIRECT-SUPERS" (classoid-name class))
              (if (typep class 'built-in-classoid)
@@ -230,27 +204,15 @@
     (mapcar (lambda (kernel-bic-entry)
               (/noshow "setting up" kernel-bic-entry)
               (let* ((name (car kernel-bic-entry))
-                     (class (find-classoid name))
-                     (prototype-form
-                      (getf (cdr kernel-bic-entry) :prototype-form)))
+                     (class (find-classoid name)))
                 (/noshow name class)
                 `(,name
                   ,(mapcar #'classoid-name (direct-supers class))
                   ,(mapcar #'classoid-name (direct-subs class))
                   ,(map 'list
-                        (lambda (x)
-                          (classoid-name
-                           (layout-classoid x)))
-                        (reverse
-                         (layout-inherits
-                          (classoid-layout class))))
-                  ,(if prototype-form
-                       (eval prototype-form)
-                       ;; This is the default prototype value which
-                       ;; was used, without explanation, by the CMU CL
-                       ;; code we're derived from. Evidently it's safe
-                       ;; in all relevant cases.
-                       42))))
+                        (lambda (x) (classoid-name (layout-classoid x)))
+                        (reverse (layout-inherits (classoid-layout class))))
+                  ,(eval (getf (cdr kernel-bic-entry) :prototype-form)))))
             (remove-if (lambda (kernel-bic-entry)
                          (member (first kernel-bic-entry)
                                  ;; remove special classes (T and our
@@ -258,7 +220,7 @@
                                  ;; BUILT-IN-CLASS list
                                  '(t function stream sequence
                                      file-stream string-stream)))
-                       sb-kernel::*!built-in-classes*))))
+                       sb-kernel::+!built-in-classes+))))
 (/noshow "done setting up SB-PCL::*BUILT-IN-CLASSES*")
 
 ;;;; the classes that define the kernel of the metabraid
@@ -398,32 +360,30 @@
 (defclass method-combination (metaobject)
   ((%documentation :initform nil :initarg :documentation)))
 
+(defun make-gf-hash-table ()
+  (make-hash-table :test 'eq
+                   :hash-function #'sb-impl::fsc-instance-hash ; stable hash
+                   :weakness :key
+                   :synchronized t))
+
 (defclass standard-method-combination (definition-source-mixin
                                        method-combination)
-  ((type-name
-    :reader method-combination-type-name
-    :initarg :type-name)
-   (options
-    :reader method-combination-options
-    :initarg :options)))
+  ((type-name :reader method-combination-type-name :initarg :type-name)
+   (options :reader method-combination-options :initarg :options)
+   (%generic-functions :initform (make-gf-hash-table)
+                       :reader method-combination-%generic-functions)))
 
 (defclass long-method-combination (standard-method-combination)
-  ((function
-    :initarg :function
-    :reader long-method-combination-function)
-   (args-lambda-list
-    :initarg :args-lambda-list
-    :reader long-method-combination-args-lambda-list)))
+  ((function :initarg :function :reader long-method-combination-function)
+   (args-lambda-list :initarg :args-lambda-list
+                     :reader long-method-combination-args-lambda-list)))
 
 (defclass short-method-combination (standard-method-combination)
-  ((operator
-    :reader short-combination-operator
-    :initarg :operator)
-   (identity-with-one-argument
-    :reader short-combination-identity-with-one-argument
+  ((operator :reader short-combination-operator :initarg :operator)
+   (identity-with-one-argument :reader short-combination-identity-with-one-argument
     :initarg :identity-with-one-argument)))
 
-(defclass slot-definition (metaobject)
+(defclass slot-definition (metaobject definition-source-mixin)
   ((name
     :initform nil
     :initarg :name
@@ -446,8 +406,7 @@
     ;; KLUDGE: we need a reader for bootstrapping purposes, in
     ;; COMPUTE-EFFECTIVE-SLOT-DEFINITION-INITARGS.
     :reader %slot-definition-documentation)
-   (%class :initform nil :initarg :class :accessor slot-definition-class)
-   (source :initform nil :initarg source :accessor definition-source)))
+   (%class :initform nil :initarg :class :accessor slot-definition-class)))
 
 (defclass standard-slot-definition (slot-definition)
   ((allocation
@@ -504,15 +463,19 @@
 ;;; these functions can access the SLOT-INFO directly, avoiding the overhead
 ;;; of accessing a standard-instance.
 (defstruct (slot-info
+            (:copier nil)
             (:constructor make-slot-info
-                (&key slotd typecheck
+                (&key slotd typecheck allocation location
                  (reader (uninitialized-accessor-function :reader slotd))
                  (writer (uninitialized-accessor-function :writer slotd))
                  (boundp (uninitialized-accessor-function :boundp slotd)))))
   (typecheck nil :type (or null function))
+  (allocation nil)
+  (location nil)
   (reader (missing-arg) :type function)
   (writer (missing-arg) :type function)
   (boundp (missing-arg) :type function))
+(declaim (freeze-type slot-info))
 
 (defclass standard-direct-slot-definition (standard-slot-definition
                                            direct-slot-definition)
@@ -580,10 +543,16 @@
                            specializer-with-object)
   ((object :initarg :object :reader specializer-object
            :reader eql-specializer-object)
+   ;; created on demand (if and when needed), the CTYPE is the representation
+   ;; of this metaobject as an internalized type object understood by the
+   ;; kernel's type machinery. The CLOS object is really just a MEMBER-TYPE,
+   ;; but the type system doesn't know that.
+   (sb-kernel:ctype)
    ;; Because EQL specializers are interned, any two putative instances
    ;; of EQL-specializer referring to the same object are in fact EQ to
    ;; each other. Therefore a list of direct methods in the specializer can
    ;; reliably track all methods that are specialized on the identical object.
+   ;; FIXME: explain why this is a cons of two NILs.
    (direct-methods :initform (cons nil nil))))
 
 ;; Why is this weak-value, not weak-key: suppose the value is unreachable (dead)
@@ -607,9 +576,8 @@
   ;; Need to lock, so that two threads don't get non-EQ specializers
   ;; for an EQL object.
   (with-locked-system-table (*eql-specializer-table*)
-    (or (gethash object *eql-specializer-table*)
-        (setf (gethash object *eql-specializer-table*)
-              (make-instance 'eql-specializer :object object)))))
+    (ensure-gethash object *eql-specializer-table*
+                    (make-instance 'eql-specializer :object object))))
 
 (defclass class (dependent-update-mixin
                  definition-source-mixin
@@ -708,7 +676,7 @@
   ((source
     :initform nil
     :reader definition-source
-    :initarg :definition-source)))
+    :initarg source)))
 
 (defclass plist-mixin (standard-object)
   ((plist :initform () :accessor object-plist :initarg plist)))
