@@ -24,15 +24,6 @@
 #include "breakpoint.h"
 #include "monitor.h"
 
-#ifdef LISP_FEATURE_LINUX
-extern int linux_sparc_siginfo_bug;
-#endif
-
-void arch_init(void)
-{
-    return;
-}
-
 os_vm_address_t arch_get_bad_addr(int sig, siginfo_t *code, os_context_t *context)
 {
 #if 1 /* New way. */
@@ -164,7 +155,6 @@ void arch_remove_breakpoint(void *pc, unsigned int orig_inst)
  * Clear?
  */
 static unsigned int *skipped_break_addr, displaced_after_inst;
-static sigset_t orig_sigmask;
 
 void arch_do_displaced_inst(os_context_t *context, unsigned int orig_inst)
 {
@@ -250,7 +240,7 @@ void
 arch_handle_single_step_trap(os_context_t *context, int trap)
 {
     unsigned int code = *((u32 *)(*os_context_pc_addr(context)));
-    int register_offset = code >> 5 & 0x1f;
+    int register_offset = code >> 8 & 0x1f;
     handle_single_step_trap(context, trap, register_offset);
     arch_skip_instruction(context);
 }
@@ -264,10 +254,11 @@ arch_handle_allocation_trap(os_context_t *context)
     int rs1;
     int size;
     int immed;
-    int context_index;
-    boolean were_in_lisp;
     char* memory;
 
+    if (foreign_function_call_active)
+      lose("Allocation trap inside foreign code.");
+
     pc = (unsigned int*) *os_context_pc_addr(context);
     or_inst = pc[-1];
 
@@ -276,8 +267,8 @@ arch_handle_allocation_trap(os_context_t *context)
      * instruction!
      */
     if (!(((or_inst >> 30) == 2) && (((or_inst >> 19) & 0x1f) == 2)))
-        lose(stderr, "Whoa!!! Got an allocation trap not preceeded by an OR inst: 0x%08x!\n",
-                or_inst);
+        lose("Allocation trap not preceded by an OR instruction: 0x%08x",
+             or_inst);
 
     /*
      * An OR instruction.  RS1 is the register we want to allocate to.
@@ -293,8 +284,6 @@ arch_handle_allocation_trap(os_context_t *context)
         size = *os_context_register_addr(context, size);
     }
 
-    if (foreign_function_call_active)
-        lose(stderr, "Whoa! allocation trap and we weren't in lisp!\n");
     fake_foreign_function_call(context);
 
     /*
@@ -304,7 +293,8 @@ arch_handle_allocation_trap(os_context_t *context)
         struct interrupt_data *data =
             arch_os_get_current_thread()->interrupt_data;
         data->allocation_trap_context = context;
-        memory = alloc(size);
+        extern lispobj alloc(sword_t);
+        memory = (char*)alloc(size);
         data->allocation_trap_context = 0;
     }
     *os_context_register_addr(context, rs1) = memory;
@@ -316,24 +306,16 @@ arch_handle_allocation_trap(os_context_t *context)
 static void sigill_handler(int signal, siginfo_t *siginfo,
                            os_context_t *context)
 {
-    if ((siginfo->si_code) == ILL_ILLOPC
-#ifdef LISP_FEATURE_LINUX
-        || (linux_sparc_siginfo_bug && (siginfo->si_code == 2))
-#endif
-        ) {
+    if ((siginfo->si_code) == ILL_ILLOPC) {
         int trap;
         unsigned int inst;
         unsigned int* pc = (unsigned int*) siginfo->si_addr;
 
         inst = *pc;
-        trap = inst & 0x1f;
+        trap = inst & 0xff;
         handle_trap(context,trap);
     }
-    else if ((siginfo->si_code) == ILL_ILLTRP
-#ifdef LISP_FEATURE_LINUX
-             || (linux_sparc_siginfo_bug && (siginfo->si_code) == 192)
-#endif
-             ) {
+    else if ((siginfo->si_code) == ILL_ILLTRP) {
         if (pseudo_atomic_trap_p(context)) {
             /* A trap instruction from a pseudo-atomic.  We just need
                to fixup up alloc-tn to remove the interrupted flag,
@@ -352,96 +334,12 @@ static void sigill_handler(int signal, siginfo_t *siginfo,
     }
 }
 
-static void sigemt_handler(int signal, siginfo_t *siginfo,
-                           os_context_t *context)
-{
-    unsigned int badinst;
-    boolean subtract, immed;
-    int rd, rs1, op1, rs2, op2, result;
-
-    badinst = *(unsigned int *)os_context_pc_addr(context);
-    if ((badinst >> 30) != 2 || ((badinst >> 20) & 0x1f) != 0x11) {
-        /* It wasn't a tagged add.  Pass the signal into lisp. */
-        interrupt_handle_now(signal, siginfo, context);
-        return;
-    }
-
-    fprintf(stderr, "SIGEMT trap handler with tagged op instruction!\n");
-
-    /* Extract the parts of the inst. */
-    subtract = badinst & (1<<19);
-    rs1 = (badinst>>14) & 0x1f;
-    op1 = *os_context_register_addr(context, rs1);
-
-    /* If the first arg is $ALLOC then it is really a signal-pending note */
-    /* for the pseudo-atomic noise. */
-    if (rs1 == reg_ALLOC) {
-        /* Perform the op anyway. */
-        op2 = badinst & 0x1fff;
-        if (op2 & (1<<12))
-            op2 |= -1<<13;
-        if (subtract)
-            result = op1 - op2;
-        else
-            result = op1 + op2;
-        /* KLUDGE: this & ~7 is a little bit magical but basically
-           clears pseudo_atomic bits if any */
-        *os_context_register_addr(context, reg_ALLOC) = result & ~7;
-        arch_skip_instruction(context);
-        interrupt_handle_pending(context);
-        return;
-    }
-
-    if ((op1 & 3) != 0) {
-        /* The first arg wan't a fixnum. */
-        interrupt_internal_error(context, 0);
-        return;
-    }
-
-    if (immed = badinst & (1<<13)) {
-        op2 = badinst & 0x1fff;
-        if (op2 & (1<<12))
-            op2 |= -1<<13;
-    }
-    else {
-        rs2 = badinst & 0x1f;
-        op2 = *os_context_register_addr(context, rs2);
-    }
-
-    if ((op2 & 3) != 0) {
-        /* The second arg wan't a fixnum. */
-        interrupt_internal_error(context, 0);
-        return;
-    }
-
-    rd = (badinst>>25) & 0x1f;
-    if (rd != 0) {
-        /* Don't bother computing the result unless we are going to use it. */
-        if (subtract)
-            result = (op1>>2) - (op2>>2);
-        else
-            result = (op1>>2) + (op2>>2);
-
-        dynamic_space_free_pointer =
-            (lispobj *) *os_context_register_addr(context, reg_ALLOC);
-
-        *os_context_register_addr(context, rd) = alloc_number(result);
-
-        *os_context_register_addr(context, reg_ALLOC) =
-            (unsigned long) dynamic_space_free_pointer;
-    }
-
-    arch_skip_instruction(context);
-}
-
 void arch_install_interrupt_handlers()
 {
-    undoably_install_low_level_interrupt_handler(SIGILL , sigill_handler);
-    undoably_install_low_level_interrupt_handler(SIGEMT, sigemt_handler);
+    undoably_install_low_level_interrupt_handler(SIGILL, sigill_handler);
 }
 
 
-#ifdef LISP_FEATURE_LINKAGE_TABLE
 
 /* This a naive port from CMUCL/sparc, which was mostly stolen from the
  * CMUCL/x86 version, with adjustments for sparc
@@ -471,8 +369,13 @@ void arch_install_interrupt_handlers()
  * Insert the necessary jump instructions at the given address.
  */
 void
-arch_write_linkage_table_jmp(void* reloc_addr, void *target_addr)
+arch_write_linkage_table_entry(int index, void *target_addr, int datap)
 {
+  char *reloc_addr = (char*)LINKAGE_TABLE_SPACE_START + index * LINKAGE_TABLE_ENTRY_SIZE;
+  if (datap) {
+    *(unsigned long *)reloc_addr = (unsigned long)target_addr;
+    return;
+  }
   /*
    * Make JMP to function entry.
    *
@@ -521,13 +424,5 @@ arch_write_linkage_table_jmp(void* reloc_addr, void *target_addr)
   *inst_ptr++ = inst;
   *inst_ptr++ = inst;
 
-  os_flush_icache((os_vm_address_t) reloc_addr, (char*) inst_ptr - (char*) reloc_addr);
+  os_flush_icache((os_vm_address_t) reloc_addr, (char*) inst_ptr - reloc_addr);
 }
-
-void
-arch_write_linkage_table_ref(void * reloc_addr, void *target_addr)
-{
-    *(unsigned long *)reloc_addr = (unsigned long)target_addr;
-}
-
-#endif
