@@ -11,7 +11,8 @@
 ;;;; absolutely no warranty. See the COPYING and CREDITS files for
 ;;;; more information.
 
-(cl:in-package :cl-user)
+(eval-when (:compile-toplevel :load-toplevel :execute)
+  (load "compiler-test-util.lisp"))
 
 (declaim (optimize (debug 3) (speed 2) (space 1)))
 
@@ -237,11 +238,12 @@
       (make-load-form-saving-slots self :slot-names *foo-save-slots*)))
 (with-test (:name :load-form-canonical-p)
   (let ((foo (make-foo :x 'x :y 'y)))
-    (assert (eq (let ((*foo-save-slots* :all)) (sb-c::%make-load-form foo))
-                'sb-fasl::fop-struct))
-    ;; specifying all slots is still canonical
-    (assert (eq (let ((*foo-save-slots* '(y x))) (sb-c::%make-load-form foo))
-                'sb-fasl::fop-struct))
+    (flet ((assert-canonical (slots)
+             (let ((*foo-save-slots* slots))
+               (assert (sb-fasl:load-form-is-default-mlfss-p foo)))))
+      (assert-canonical :all)
+      (assert-canonical '(x y)) ; specifying all slots explicitly is still canonical
+      (assert-canonical '(y x)))
     ;; specifying only one slot is not canonical
     (assert (equal (let ((*foo-save-slots* '(x))) (sb-c::%make-load-form foo))
                    '(sb-kernel::new-instance foo)))))
@@ -254,11 +256,11 @@
 ;; a tangled forest of structures,
 (defparameter *metadata* '#.(compute-tangled-stuff))
 
-(test-util:with-test (:name :make-load-form-huge-vector)
+(with-test (:name :make-load-form-huge-vector)
   (assert (equalp (compute-airports (length (the vector *airport-vector*)))
                   *airport-vector*)))
 
-(test-util:with-test (:name :make-load-form-circular-hair)
+(with-test (:name :make-load-form-circular-hair)
   (let ((testcase (compute-tangled-stuff)))
     (declare (optimize (speed 1)))
     ;; MAKE-LOAD-FORM discards the value of the CDF slot of one structure.
@@ -334,8 +336,10 @@
 ;; Track the make-load-form FOPs as they fly by at load-time.
 (defvar *call-tracker* nil)
 (dolist (fop-name 'sb-fasl::(fop-allocate-instance fop-set-slot-values))
-  (let* ((index (position fop-name sb-fasl::**fop-names**))
-         (fun (the function (aref sb-fasl::**fop-funs** index))))
+  (let* ((index (position fop-name sb-fasl::**fop-funs**
+                          :key
+                          (lambda (x) (and (functionp x) (sb-kernel:%fun-name x)))))
+         (fun (aref sb-fasl::**fop-funs** index)))
     (setf (aref sb-fasl::**fop-funs** index)
           (lambda (&rest args)
             (push fop-name *call-tracker*)
@@ -397,3 +401,48 @@
 ;; This tests the edge case: that we DO call slot-makunbound.
 (with-test (:name :mlfss-slot-makunbound)
   (assert (not (slot-boundp *fool1* 'a-slot))))
+
+(defun try-literal-layout () #.(sb-kernel:find-layout 'class-with-shared-slot))
+(with-test (:name :dump-std-obj-literal-layout)
+  (assert (eq (try-literal-layout)
+              (sb-kernel:find-layout 'class-with-shared-slot))))
+
+(defparameter *some-hash-table*
+  #.(let ((ht (make-hash-table)))
+      (setf (gethash 'first ht) :a)
+      (setf (gethash 'second ht) :b)
+      ht))
+;;; In the interest of producing repeatable results from externalized
+;;; hash-tables, the make-load-form method iterates in such a way that
+;;; the k/v vector should be ordered the same as it originally was.
+;;; This also has implications on the order of items in each bucket
+;;; when there are collisions.
+(with-test (:name :reconstructed-hash-table)
+  (let ((pairs (sb-impl::hash-table-pairs *some-hash-table*)))
+    (assert (eq (aref pairs 2) 'first))
+    (assert (eq (aref pairs 4) 'second))))
+
+(defun list-coalescing-test-fun-1 ()
+  ;; base coalesces with base, non-base coalesces with non-base
+  (values '#.`(foo ,(coerce "a" 'base-string))
+          '#.`(foo ,(coerce "a" '(array character)))
+          '#.`(foo ,(coerce "a" 'base-string))
+          '#.`(foo ,(coerce "a" '(array character)))))
+
+(defun list-coalescing-test-fun-2 ()
+  (values '#.`(foo ,(coerce "a" '(array character)))
+          '#.`(foo ,(coerce "a" 'base-string))))
+
+(with-test (:name :more-code-constant-coalescing
+                  :skipped-on (:not :sb-unicode))
+  (let ((l1 (ctu:find-code-constants #'list-coalescing-test-fun-1))
+        (l2 (ctu:find-code-constants #'list-coalescing-test-fun-2)))
+    (assert (equal (length l1) 2))
+    (assert (equal (length l2) 2))
+    (multiple-value-bind (c1 c2 c3 c4) (list-coalescing-test-fun-1)
+      (assert (typep (second c1) 'simple-base-string))
+      (assert (typep (second c2) 'sb-kernel:simple-character-string))
+      (assert (typep (second c3) 'simple-base-string))
+      (assert (typep (second c4) 'sb-kernel:simple-character-string)))
+    (assert (or (equal l1 l2)
+                (equal l1 (reverse l2))))))
