@@ -133,6 +133,11 @@ echo 'int bar() { return 42; }' >> $TEST_FILESTEM-b.c
 build_so $TEST_FILESTEM-b
 
 echo 'int foo = 42;' > $TEST_FILESTEM-b2.c
+# Try not to produce two more-or-less physically identical '.so' files (modulo
+# some constants in the functions), so that the test will fail if we accidentally
+# call the new bar() at the old address - as might happen if the OS maps both '.so'
+# files at the same place - as a consequence of failing to update the linkage table.
+echo 'int somerandomfun(int x) { return x?-x:bar(); }' >> $TEST_FILESTEM-b2.c
 echo 'int bar() { return 13; }' >> $TEST_FILESTEM-b2.c
 build_so $TEST_FILESTEM-b2
 
@@ -264,7 +269,6 @@ cat > $TEST_FILESTEM.test.lisp <<EOF
   ;; test late resolution
   (eval-when (:compile-toplevel :load-toplevel :execute)
    (setq *features* (union *features* sb-impl:+internal-features+)))
-  #+linkage-table
   (progn
     (note "/starting linkage table tests")
     (define-alien-variable late-foo int)
@@ -320,7 +324,7 @@ test_save() {
     run_sbcl --load $TEST_FILESTEM.$1.fasl <<EOF
 (eval-when (:compile-toplevel :load-toplevel :execute)
  (setq *features* (union *features* sb-impl:+internal-features+)))
-#+linkage-table (save-lisp-and-die "$TEST_FILESTEM.$x.core")
+(save-lisp-and-die "$TEST_FILESTEM.$x.core")
 (sb-ext:exit :code 22) ; catch this
 EOF
     check_status_maybe_lose "save $1" $? \
@@ -359,7 +363,7 @@ if [ -f $TEST_FILESTEM.fast.core ] ; then
               (invoke-restart cont)))
           (print :fell-through)
           (invoke-debugger condition)))
-   #+linkage-table (save-lisp-and-die "$TEST_FILESTEM.missing.core")
+   (save-lisp-and-die "$TEST_FILESTEM.missing.core")
    (sb-ext:exit :code 22) ; catch this
 EOF
     check_status_maybe_lose "saving-missing-so-core" $? \
